/*
** input.c (C) 2013 Josh PH3NOM Pearson
**
** An input callback to update the camera position and destination
** based on user input in a 3d environment. Only meant for testing.
*/

#include <kos.h>

#include "vector.h"
#include "input.h"

#define RANGLE 15.0f          // Rotation Angle
#define RSD 360.0f/RANGLE     // Rotated Side
static const float ANGLE = 10.0f;
#define ROTATION RADIAN*ANGLE
static const float RSHIFT = RADIAN*90.0f;
static const float FRSHIFT = RADIAN*270.0f;

int InputCallback2(Player * p)
{
     maple_device_t *cont = maple_enum_type(0, MAPLE_FUNC_CONTROLLER);    

     if (cont)
     {          
        cont_state_t *state = (cont_state_t *)maple_dev_status(cont);
         
        if(!state)
           return 0;

		if(state->buttons & CONT_DPAD_UP || state->buttons & CONT_DPAD2_UP) 
           if(p->lookAt.y <= p->position.y + 90.0f)
              p->lookAt.y += 0.5f * 2.0f;
		
		if(state->buttons & CONT_DPAD_DOWN || state->buttons & CONT_DPAD2_DOWN) 
           if(p->lookAt.y >= p->position.y - 90.0f)
              p->lookAt.y -= 0.5f * 2.0f;
       
		if(state->buttons & CONT_DPAD_LEFT || state->buttons & CONT_DPAD2_LEFT) 
	       vec3f_rotr_xz(p->lookAt.x, p->lookAt.y, p->lookAt.z,
		                 p->position.x, p->position.y, p->position.z, -ROTATION/2.0 * 2.0f);
	       
		if (state->buttons & CONT_DPAD_RIGHT || state->buttons & CONT_DPAD2_RIGHT) 
	       vec3f_rotr_xz( p->lookAt.x, p->lookAt.y, p->lookAt.z,
		                  p->position.x, p->position.y, p->position.z, ROTATION/2.0 * 2.0f);

		if (state->buttons & CONT_Y) 
           vec3f_shift_xz(&p->lookAt, &p->position, 1.0f * 2.0f);
        
        if (state->buttons & CONT_A) 
           vec3f_shift_xz(&p->lookAt, &p->position, -1.0f * 2.0f);
		
        if (state->buttons & CONT_X) 
	    {            
	       vec3f_rotr_xz(p->lookAt.x, p->lookAt.y, p->lookAt.z,
		                 p->position.x, p->position.y, p->position.z, RSHIFT * 2.0f);
		   vec3f_shift_xz(&p->lookAt, &p->position, -1.0f  * 2.0f);
	       vec3f_rotr_xz(p->lookAt.x, p->lookAt.y, p->lookAt.z,
		                 p->position.x, p->position.y, p->position.z, FRSHIFT * 2.0f);
        }   
		
        if (state->buttons & CONT_B) 
	    {
	       vec3f_rotr_xz(p->lookAt.x, p->lookAt.y, p->lookAt.z,
		                 p->position.x, p->position.y, p->position.z, RSHIFT * 2.0f);              
           vec3f_shift_xz(&p->lookAt, &p->position, 1.0f  * 2.0f);
	       vec3f_rotr_xz(p->lookAt.x, p->lookAt.y, p->lookAt.z,
		                 p->position.x, p->position.y, p->position.z, FRSHIFT * 2.0f);
        }    

		if(state->ltrig > 0.0f)
		{
			if(state->buttons & CONT_DPAD_UP)
				p->state &= ~PSTATE_DUCK;

			if(state->buttons & CONT_DPAD_DOWN)
				p->state |= PSTATE_DUCK;
		}
	 }
	 return 0;
}
