/* 
** OpenGL Quake 3 BSP Render Example (C) 2013-2014 Josh PH3NOM Pearson
*/

#include <kos.h>

#include <time.h>
#include <arch/timer.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "input.h"
#include "font.h"
#include "player.h"
#include "q3-bsp.h"
#include "texture.h"
#include "vector.h"

static vec3f_t up = { 0, 1, 0 };

static GLuint texture[8];

static Font *font;

static Player player1;

/* Get current hardware timing using arch/timer.h */
static uint32 s, ms;
static uint64 msec;

unsigned int GetTime()
{
   	timer_ms_gettime(&s, &ms);
	msec = (((uint64)s) * ((uint64)1000)) + ((uint64)ms);
	return (unsigned int)msec;
}

void R_Player1()
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0f, 16.0f / 9.0f, 0.1f, 10000.0f);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glViewport(0, 0, vid_mode->width, vid_mode->height);

	gluLookAt(player1.position.x, player1.position.y, player1.position.z,
		      player1.lookAt.x, player1.lookAt.y, player1.lookAt.z,
			  up.x, up.y, up.z);

	glEnable(GL_KOS_NEARZ_CLIPPING);

    Q3BSP_RenderArraysMultiTex();

	glDisable(GL_KOS_NEARZ_CLIPPING);
}

void R_Callback( )
{    
	char timeString[32] = { '\n' };
	
	unsigned int FRAME_START = GetTime();

	R_Player1();

    GLuint CPU_TIME = GetTime() - FRAME_START;
    
    FRAME_START = GetTime();
    
    glutSwapBuffers();

    GLuint GPU_TIME = GetTime() - FRAME_START;
    
    sprintf(timeString,"FRAME TIME: CPU: %i || GPU: %i\n", CPU_TIME, GPU_TIME);
}

static void DCE_InitKGL()
{
    glKosInit();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0f, vid_mode->width / vid_mode->height, 0.1f, 100000.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glShadeModel(GL_SMOOTH);
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);   
}

static int DCE_LoadQ3BSP(char * bsp)
{
    Q3BSP_Verbose( 1 );

    if(!Q3BSP_Load(bsp, &player1.position))
    {
	    printf("ERROR Parsing BSP!\n");
	    return 0;
    }
    
	Q3BSP_CacheArraysMultiTex();

	return 1;
}

static void DCE_Init()
{
	DCE_InitKGL(); 

	texture[0] = glTextureLoadPVR("/rd/font/FONT0.pvr", 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_FILTER, GL_LINEAR);
    font = FontInit(512, 512, 10, 10, 0x000000FF);    

	player1.position.x = -140;
	player1.position.y = 80;
	player1.position.z = -315;

	player1.lookAt.x = -140;
	player1.lookAt.y = 80;
	player1.lookAt.z = -305;

	player1.height = 80;
	player1.state = PSTATE_IDLE;
	player1.lstate = PSTATE_IDLE;
	player1.state_frame = 0;
}

extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);		   

int main(int argc, char **argv)
{
	DCE_Init();

	DCE_LoadQ3BSP("/cd/maps/test1.bsp");

    while(1)
	{
		InputCallback2(&player1);

        R_Callback();
    }

    return 0;
}
