//=======================================================================================//
// Q3-BSP.H (C) 2014 Josh PH3NOM Pearson
//=======================================================================================//

#ifndef Q3_BSP_H
#define Q3_BSP_H

#include "vector.h"

typedef unsigned char byte;
typedef signed   int sint32;

#ifndef BOOL_IS_DEFINED
#define BOOL_IS_DEFINED
typedef unsigned char bool;
#define true  1
#define false 0
#endif

#ifndef VECTOR3F_IS_DEFINED
#define VECTOR3F_IS_DEFINED

typedef float float2x2[2][2];
typedef float float2x3[2][3];

typedef struct
{
	float x, y, z, w;
} vector4f;

#endif

//=================================================================//

#define Q3_BSP_HDR_VERSION   0x2e

#define	Q3_LUMP_ENTITIES	 0x00
#define	Q3_LUMP_TEXTURES	 0x01
#define	Q3_LUMP_PLANES	     0x02
#define	Q3_LUMP_NODES        0x03
#define	Q3_LUMP_LEAFS        0x04
#define	Q3_LUMP_LEAF_FACES   0x05
#define	Q3_LUMP_LEAF_BRUSHES 0x06
#define	Q3_LUMP_MODELS       0x07
#define	Q3_LUMP_BRUSHES 	 0x08
#define	Q3_LUMP_BRUSH_SIDES  0x09
#define	Q3_LUMP_VERTICES     0x0a
#define	Q3_LUMP_FACE_INDICES 0x0b
#define	Q3_LUMP_SHADERS      0x0c
#define	Q3_LUMP_FACES    	 0x0d
#define	Q3_LUMP_LIGHT_MAPS   0x0e
#define	Q3_LUMP_LIGHT_VOLS   0x0f
#define Q3_LUMP_VIS_DATA     0x10

#define Q3_BSP_HDR_LUMPS     0x11

#define Q3_ENT_START         0x7B
#define Q3_ENT_SEP           0x0A
#define Q3_ENT_QUOTE         0x22
#define Q3_ENT_SPACE         0x20
#define Q3_ENT_END           0x7D

typedef struct
{
	int oft;
	int len;
} BSP_LUMP;

typedef struct
{
	char id[4];
	int  version;
	BSP_LUMP lump[Q3_BSP_HDR_LUMPS];
} Q3_BSP_HDR;

typedef struct
{
	byte *lump[Q3_BSP_HDR_LUMPS];
} Q3_BSP_LUMPS;

typedef struct
{
    vec3f_t pos;
    vector4f uv;
    vec3f_t normal;
    unsigned int color; //RGBA
} Q3_BSP_VERTEX;

typedef struct
{
    uint32 textureID;
    sint32 effect;
    uint32 type;
    uint32 vertexIndex, vertices;
    uint32 faceIndex, faceIndices;
    uint32 lightmapID;
    uint32 lmStart[2];
    uint32 lmSize[2];
    vec3f_t lmOrigin;
    float2x3 lmVector;
    vec3f_t normal;
    uint32 size[2];
} Q3_BSP_FACE;

typedef struct
{
	char name[64];
	int  flags;
	int  contents;
} Q3_BSP_TEXTURE;

typedef struct
{
	vec3f_t normal;
	float intercept;
} Q3_BSP_PLANE;

typedef struct
{
	int planeIndex;
	int front, back;	//child nodes
	int mins[3];	    //min x,y,z (bounding box)
	int maxs[3];        //max x,y,z (bounding box)
} Q3_BSP_NODE;

typedef struct
{
	int clusters;
	int bytesPerCluster;
	byte *bitset;
} Q3_BSP_VIS_DATA;

typedef struct
{
	int cluster;	//cluster index for visdata
	int area;		//areaportal area
	int mins[3];	//min x,y,z (bounding box)
	int maxs[3];
	int firstLeafFace;	//first index in leafFaces array
	int numLeafFaces;
	int firstLeafBrush;	//first index into leaf brushes array
	int numLeafBrushes;
} Q3_BSP_LEAF;

typedef struct 
{
	int	nFirstSide;		// First brushside for brush
    int NumSides;		// Number of brushsides for brush
    int nIndex;			// Texture index
} Q3_BSP_BRUSH;
 
typedef struct 
{
	int nPlaneIndex;	// Lookup for plane
	int nIndex;			// Texture index
} Q3_BSP_BRUSH_SIDE;

typedef char Q3_BSP_LIGHTMAP[128*128*3];     // RGB 24bit
typedef char Q3_BSP_LIGHTMAP_DC[128*128*2];  // RGB 16bit for use with PVR_TXRFMT_RGB565

//=====================================================//

void Q3BSP_Verbose(byte enable);

bool Q3BSP_Load(char *fname, vec3f_t *startPos);

void Q3BSP_Render();

void Q3BSP_CalculateVisibleFaces(vec3f_t *camera, vec3f_t *camto);

void Q3BSP_RenderVisibleFaces();

void Q3BSP_CacheFaceVertices();

void Q3BSP_RenderVisibleFaceVertices();

void Q3BSP_LoadIndices();

void Q3BSP_AssembleElements(vec3f_t *camera, vec3f_t *camto);

void Q3BSP_RenderImmediate();

void Q3BSP_RenderImmediateTextured();

void Q3BSP_RenderArrays();

void Q3BSP_RenderElements();

void Q3BSP_RenderArraysMultiTex();
void Q3BSP_CacheArraysMultiTex();

void Q3BSP_CacheVAO(void);
void Q3BSP_RenderVAO(void);

unsigned char Q3BSP_CheckCollision(vec3f_t * vStart, vec3f_t * vEnd, vec3f_t * vHit);

#endif