/*
** vector.c (C) 2013 Josh PH3NOM Pearson
**
** A set of functions for working with 3D Vectors optimized for the SH4
*/

#ifndef VECTOR_H
#define VECTOR_H

#include <GL/gl.h>
#include <dc/vec3f.h>

#define DEG2RAD (F_PI / 180.0f)
#define RAD2DEG (180.0f / F_PI)
#define RADIAN 0.0174532925   // Convert Degrees to Radians
#define CIRCLE 6.2831853      // RADIAN * 360.0f

void vec3f_lerp(vec3f_t * v1, vec3f_t * v2, vec3f_t * out, float mag);
void vec3f_copy(vec3f_t * src, vec3f_t * dst);
void vec3f_shift_xz(vec3f_t * p, vec3f_t * c, float mag);
void vec3f_shift(vec3f_t * p, vec3f_t * c, float mag) ;
void vec3f_sub(vec3f_t * v1, vec3f_t * v2, vec3f_t * v3);
void vec3f_add(vec3f_t * v1, vec3f_t * v2, vec3f_t * v3);
void vec3f_mul(vec3f_t * v1, vec3f_t * v2, vec3f_t * v3);

#ifndef max
#define max(a,b) a>b ? a : b
#endif
#ifndef min
#define min(a,b) a<b ? a : b
#endif

/* SH4 fmac - Returns a*b+c at the cost of a single floating-point operation */
#define fmac( a, b, c ) ({ \
     register float __FR0 __asm__("fr0") = a; \
     register float __FR1 __asm__("fr1") = b; \
     register float __FR2 __asm__("fr2") = c; \
     __asm__ __volatile__( \
        "fmac   fr0, fr1, fr2\n" \
        : "=f" (__FR0), "=f" (__FR1), "=f" (__FR2) \
        : "0" (__FR0), "1" (__FR1), "2" (__FR2) \
        ); \
     __FR2; })

#endif
